package de.fhapp.jme.scrap;

import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.sound.midi.InvalidMidiDataException;
import javax.sound.midi.MidiDevice;
import javax.sound.midi.MidiDevice.Info;
import javax.sound.midi.MidiSystem;
import javax.sound.midi.MidiUnavailableException;
import javax.sound.midi.Receiver;
import javax.sound.midi.ShortMessage;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;

public class MidiExample {
	JFrame frm;
	JLabel lblChannel;
	JTextField txtChannel;
	private JTextArea outArea;
	private JLabel lblNoteValue;
	private JTextField txtNoteValue;
	private JLabel lblVelocity;
	private JTextField txtVelocity;
	private JLabel lblNoteLength;
	private JTextField txtNoteLength;
	private JButton btnListDevices;
	private JLabel lblDeviceIndex;
	private JTextField txtDeviceIndex;
	private Component btnConnect;
	
	MidiDevice currentDev = null;
	Receiver rcvr;
	private JButton btnSendNote;
	
	private JLabel buildLabel(String labelText) {
		final Dimension theDim = new Dimension(150,25);
		JLabel lbl = new JLabel(labelText);
		lbl.setPreferredSize(theDim);
		return lbl;
	}
	
	private JTextField buildTextField(String defaultText) {
		final Dimension theDim = new Dimension(40,25);
		if (defaultText==null) defaultText = "";
		JTextField txt = new JTextField(defaultText);
		txt.setPreferredSize(theDim);
		return txt;
	}
	
	private JButton buildButton (String buttonText, int action) {
		final Dimension theDim = new Dimension(100,25);
		JButton btn = new JButton (buttonText);
		btn.setPreferredSize(theDim);
		btn.addActionListener(new myAL(action));
		return btn;
	}
	
	private void output(String theText) {
		outArea.append(theText);
		outArea.append("\n");
	}
	
	private void connectDevice() {
		Info[] devInfos = MidiSystem.getMidiDeviceInfo();
		try {
			int devIndex = Integer.parseInt(txtDeviceIndex.getText());
			currentDev = MidiSystem.getMidiDevice(devInfos[devIndex]);
			if(!currentDev.isOpen()) currentDev.open();
			if(currentDev.getMaxReceivers() == 0) {
				output("The device " + devInfos[devIndex].getName() + " is not a receiver.");
				return;
			}
			rcvr = currentDev.getReceiver();
			output("Connected to " + devInfos[devIndex].getName());
		}
		catch (NumberFormatException ne) {
			output(ne.getLocalizedMessage());
		} catch (MidiUnavailableException mue) {
			output(mue.getLocalizedMessage());
		}
	}
	
	private void sendNote() {
		ShortMessage sm = new ShortMessage();
		try {
			int channel = Integer.parseInt(txtChannel.getText());
			int noteVal = Integer.parseInt(txtNoteValue.getText());
			int velo = Integer.parseInt(txtVelocity.getText());
			int noteLength = Integer.parseInt(txtNoteLength.getText());
			sm.setMessage(ShortMessage.NOTE_ON, channel-1, noteVal, velo);
			rcvr.send(sm, -1);
			Thread.sleep(noteLength);
			sm.setMessage(ShortMessage.NOTE_OFF, channel-1, noteVal, velo);
			
		}
		catch (NumberFormatException ne) {
			output(ne.getLocalizedMessage());
		} catch (InvalidMidiDataException imde) {
			output(imde.getLocalizedMessage());
		} catch (InterruptedException ie) {
			output(ie.getLocalizedMessage());
		}
	}
	
	private class myAL implements ActionListener {
		private int action;
		private myAL (int action) {
			this.action = action;
		}
		@Override
		public void actionPerformed(ActionEvent arg0) {
			switch(action) {
			case 0 : output(ListDevices.listMidiDevices());break;
			case 1 : connectDevice();break;
			case 3 : sendNote(); break;
			
			default : output("Unknown action " + action + "\n");
			}
			
		}
	}
	
	private class myWL extends WindowAdapter {
		@Override
		public void windowClosing(WindowEvent arg0) {
			System.exit(0);
		}
	}
	
	private void buildFrame() {
		frm = new JFrame();
		frm.setSize(280, 600);
		frm.setTitle("Midi Example");
		frm.addWindowListener(new myWL());
		Container cont = frm.getContentPane();
		cont.setLayout(new FlowLayout());
		btnListDevices = buildButton("list", 0);
		cont.add(btnListDevices);
		btnConnect = buildButton("Connect",1);
		cont.add(btnConnect);
		lblDeviceIndex = buildLabel("Device index:");
		cont.add(lblDeviceIndex);
		txtDeviceIndex = buildTextField("0");
		cont.add(txtDeviceIndex);
		lblChannel = buildLabel("Channel (1-16):");
		cont.add(lblChannel);
		txtChannel = buildTextField("1");
		cont.add(txtChannel);
		lblNoteValue = buildLabel("Note value:");
		cont.add(lblNoteValue);
		txtNoteValue = buildTextField("60");
		cont.add(txtNoteValue);
		lblVelocity = buildLabel("Velocity:");
		cont.add(lblVelocity);
		txtVelocity = buildTextField("80");
		cont.add(txtVelocity);
		lblNoteLength = buildLabel("Note length (ms):");
		cont.add(lblNoteLength);
		txtNoteLength = buildTextField("1000");
		cont.add(txtNoteLength);
		btnSendNote = buildButton("Send note",3);
		cont.add(btnSendNote);
		outArea = new JTextArea();
		JScrollPane sp = new JScrollPane(outArea);
		sp.setPreferredSize(new Dimension (260,400));
		cont.add(sp);
		
		
		frm.setVisible(true);
	}
	
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		MidiExample me = new MidiExample();
		me.buildFrame();
	}

}
